package com.fjhx.form.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.utils.Assert;
import com.fjhx.common.utils.excel.util.ExcelUtil;
import com.fjhx.customer.entity.customer.po.Customer;
import com.fjhx.customer.entity.customer.vo.CustomerVo;
import com.fjhx.customer.service.customer.CustomerService;
import com.fjhx.form.entity.*;
import com.fjhx.form.entity.customer.po.CustomerPayableBalance;
import com.fjhx.form.mapper.ReportMapper;
import com.fjhx.form.service.ReportService;
import com.fjhx.form.service.customer.CustomerPayableBalanceService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.report.po.ReportLossesDetails;
import com.fjhx.mes.service.report.ReportLossesDetailsService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class ReportServiceImpl implements ReportService {

    private final ReportMapper reportMapper;
    private final ReportLossesDetailsService reportLossesDetailsService;
    private final ProductInfoService productInfoService;
    private final ISysDeptService deptService;

    private final CustomerPayableBalanceService customerPayableBalanceService;

    private final CustomerService customerService;
    private final ISysUserService userService;

    @Autowired
    public ReportServiceImpl(ReportMapper reportMapper, ReportLossesDetailsService reportLossesDetailsService, ProductInfoService productInfoService, ISysDeptService sysDeptService, CustomerPayableBalanceService customerPayableBalanceService, CustomerService customerService, ISysUserService userService) {
        this.reportMapper = reportMapper;
        this.reportLossesDetailsService = reportLossesDetailsService;
        this.productInfoService = productInfoService;
        this.deptService = sysDeptService;
        this.customerPayableBalanceService = customerPayableBalanceService;
        this.customerService = customerService;
        this.userService = userService;
    }

    @Override
    public Page<ProductionReportBo> productionReport(ProductionReportSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        Page<ProductionReportBo> page = reportMapper.productionReport(dto.getPage(), wrapper);
        List<ProductionReportBo> productionReportBos = page.getRecords();
        List<Long> ptIds = productionReportBos.stream().map(ProductionReportBo::getId).collect(Collectors.toList());


        //获取部门列表
        List<SysDept> companyList = deptService.list();
        Map<Long, String> companyMap = companyList.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //获取报损信息
        List<ReportLossesDetails> reportLossesDetails = reportLossesDetailsService.list(q -> q.in(ReportLossesDetails::getProdTaskId, ptIds));
        Map<Long, List<ReportLossesDetails>> reportLossesMap = reportLossesDetails.stream().collect(Collectors.groupingBy(ReportLossesDetails::getProdTaskId));

        for (ProductionReportBo vo : productionReportBos) {
            vo.setFactoryName(companyMap.get(vo.getFactoryId()));

            List<ReportLossesDetails> reportLossesList = reportLossesMap.getOrDefault(vo.getId(), new ArrayList<>());
            //原材料损耗 数量，成本
            List<ReportLossesDetails> rawReportLosses = reportLossesList.stream()
                    .filter(item -> ObjectUtil.equals(item.getMaterialType(), 1)).collect(Collectors.toList());
            BigDecimal rawLossCount = rawReportLosses.stream().map(ReportLossesDetails::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal rawLossAmount = rawReportLosses.stream().map(ReportLossesDetails::getMaterialPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            vo.setRawLossCount(rawLossCount);
            vo.setRawLossAmount(rawLossAmount);
            //普通物料 损耗 数量，成本
            List<ReportLossesDetails> bomReportLosses = reportLossesList.stream()
                    .filter(item -> ObjectUtil.notEqual(item.getMaterialType(), 1)).collect(Collectors.toList());
            BigDecimal bomLossCount = bomReportLosses.stream().map(ReportLossesDetails::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal bomLossAmount = bomReportLosses.stream().map(ReportLossesDetails::getMaterialPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            vo.setBomLossCount(bomLossCount);
            vo.setBomLossAmount(bomLossAmount);
            //返工 道数，成本
            Integer reworkCount = bomReportLosses.stream().map(ReportLossesDetails::getReCount).reduce(Integer::sum).orElse(0);
            BigDecimal reworkAmount = bomReportLosses.stream().map(ReportLossesDetails::getRePrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            vo.setReworkCount(reworkCount);
            vo.setReworkAmount(reworkAmount);
            //出错率
            BigDecimal multiply = vo.getRawLossCount().divide(vo.getQuantity(), 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100));
            vo.setErrRate(multiply);
        }

        return page;
    }

    @Override
    public Page<SaleReportBo> saleReport(ProductionReportSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        Page<SaleReportBo> saleReportBoPage = reportMapper.saleReport(dto.getPage(), wrapper);
        List<SaleReportBo> records = saleReportBoPage.getRecords();

        //获取部门列表
        List<SysDept> deptList = deptService.list();
        Map<Long, String> deptMap = deptList.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        for (SaleReportBo record : records) {
            //业务公司
            record.setOfCompanyName(deptMap.get(record.getOfCompanyId()));
            //业务部门
            record.setDeptName(deptMap.get(record.getDeptId()));
            //工厂
            record.setCompanyName(deptMap.get(record.getCompanyId()));
            //不及时率
            BigDecimal untimelyRate = record.getOutInterval().divide(BigDecimal.valueOf(30), 2, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100));
            record.setUntimelyRate(untimelyRate);
        }
        return saleReportBoPage;
    }

    @Override
    public List<WaitShipmentReportStatisticBo> waitShipmentReportStatistic() {
        List<WaitShipmentReportStatisticBo> records = reportMapper.waitShipmentReportStatistic();

        //赋值业务员
        UserUtil.assignmentNickName(records, WaitShipmentReportStatisticBo::getSalesmanId, WaitShipmentReportStatisticBo::setSalesmanName);

        return records;
    }

    /**
     * 已完工未出货报表
     */
    @Override
    public Page<WaitShipmentReportBo> waitShipmentReport(ProductionReportSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        Page<WaitShipmentReportBo> waitShipmentReportBoPage = reportMapper.waitShipmentReport(dto.getPage(), wrapper);
        if (ObjectUtil.isEmpty(waitShipmentReportBoPage)) {
            return waitShipmentReportBoPage;
        }
        List<WaitShipmentReportBo> records = waitShipmentReportBoPage.getRecords();
        for (WaitShipmentReportBo record : records) {
            //超期修正
            if (ObjectUtil.isEmpty(record.getOverdueDay()) || record.getOverdueDay() < 0) {
                record.setOverdueDay(0);
            }
        }

        //赋值业务员
        UserUtil.assignmentNickName(records, WaitShipmentReportBo::getSalesmanId, WaitShipmentReportBo::setSalesmanName);

        //赋值产品信息
        productInfoService.attributeAssign(records, WaitShipmentReportBo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCustomCode());
            item.setProductName(productInfo.getName());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());
        });

        return waitShipmentReportBoPage;
    }

    /**
     * 订单汇总查询条件
     */
    private void setOrderSummaryWrapper(OrderSummarySelectDto dto, IWrapper<Object> wrapper) {
        if (ObjectUtil.isNotEmpty(dto.getAccountAge())) {
            wrapper.having("accountAge = {0}", dto.getAccountAge());
        }
        if (ObjectUtil.isNotEmpty(dto.getIsSettlement())) {
            wrapper.having("isSettlement = {0}", dto.getIsSettlement());
        }
        wrapper.keyword(dto.getKeyword(),
                new SqlField("c.code"),
                new SqlField("cu.name"),
                new SqlField("cu.short_name")
        );
        wrapper.orderByDesc("c.create_time", "c.id");
    }

    /**
     * 订单汇总数据处理
     */
    private void setOrderSummaryInfo(List<OrderSummaryBo> records) {
        UserUtil.assignmentNickName(records, OrderSummaryBo::getSalesmanId, OrderSummaryBo::setSalesmanName);
        UserUtil.assignmentNickName(records, OrderSummaryBo::getMerchUserId, OrderSummaryBo::setMerchUserName);
    }

    @Override
    public Page<OrderSummaryBo> orderSummary(OrderSummarySelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        //赋值查询条件
        setOrderSummaryWrapper(dto, wrapper);
        Page<OrderSummaryBo> page = reportMapper.orderSummary(dto.getPage(), dto, wrapper);
        List<OrderSummaryBo> records = page.getRecords();
        //赋值信息
        setOrderSummaryInfo(records);
        return page;
    }

    @Override
    public void orderSummaryExcelExport(OrderSummarySelectDto dto, HttpServletResponse httpServletResponse) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        //赋值查询条件
        setOrderSummaryWrapper(dto, wrapper);
        List<OrderSummaryBo> records = reportMapper.orderSummary(dto, wrapper);
        //赋值信息
        setOrderSummaryInfo(records);

        //Excel数据处理
        for (OrderSummaryBo record : records) {
            //订单类型处理
            String contractType = record.getContractType();
            switch (contractType) {
                case "1":
                    contractType = "外贸订单（退税）";
                    break;
                case "2":
                    contractType = "内销订单";
                    break;
                case "3":
                    contractType = "打样订单";
                    break;
                case "4":
                    contractType = "外贸订单（不退税）";
                    break;
            }
            record.setContractType(contractType);

            //订单性质处理
            String belongType = record.getBelongType();
            switch (belongType) {
                case "1":
                    belongType = "归属工厂";
                    break;
                case "2":
                    belongType = "归属业务";
                    break;
            }
            record.setBelongType(belongType);

            //是否可结算
            String isSettlement = record.getIsSettlement();
            switch (isSettlement) {
                case "0":
                    isSettlement = "否";
                    break;
                case "1":
                    isSettlement = "是";
                    break;
            }
            record.setIsSettlement(isSettlement);

            //是否已结算处理
            String settleStart = DateUtil.format(record.getSettleStartDate(), "yyyy-MM-dd");
            String settleEnd = DateUtil.format(record.getSettleEndDate(), "yyyy-MM-dd");
            record.setSettleDateStr(String.format("%s~%s ", ObjectUtil.isEmpty(settleStart) ? "" : settleStart, ObjectUtil.isEmpty(settleEnd) ? "" : settleEnd));

            //出货日期处理
            record.setFirstTruckDateStr(DateUtil.format(record.getFirstTruckDate(), "yyyy-MM-dd"));
        }

        //导出Excel
        ExcelUtil.export(httpServletResponse, records, OrderSummaryBo.class);
    }

    /**
     * 应收余额核对
     */
    @Override
    public List<ReceivableCheckBo> receivableCheck(ReceivableCheckSelectDto dto) {
        Assert.notEmpty(dto.getCheckDate(), "核对日期不能为空！");
        dto.setCheckDate(DateUtil.endOfMonth(dto.getCheckDate()));

        IWrapper<Object> wrapper = IWrapper.getWrapper();
        wrapper.keyword(dto.getKeyword(), new SqlField("cu.name"), new SqlField("cu.short_name"));
        wrapper.eq("cu.user_id", dto.getSaleUserId());

        //帐龄过滤
        wrapper.ne(ObjectUtil.equals(dto.getAccountAge(), "0~30"), "c1.ageLess30Amount", 0);
        wrapper.ne(ObjectUtil.equals(dto.getAccountAge(), "30~60"), "c1.age30To60Amount", 0);
        wrapper.ne(ObjectUtil.equals(dto.getAccountAge(), "60~90"), "c1.age60To90Amount", 0);
        wrapper.ne(ObjectUtil.equals(dto.getAccountAge(), "90~120"), "c1.age90To120Amount", 0);
        wrapper.ne(ObjectUtil.equals(dto.getAccountAge(), "120以上"), "c1.ageGreater120Amount", 0);

        List<ReceivableCheckBo> list = reportMapper.receivableCheck(dto, wrapper);
        UserUtil.assignmentNickName(list, ReceivableCheckBo::getSaleUserId, ReceivableCheckBo::setSaleUserName);
        //添加总计行
        ReceivableCheckBo sumBo = new ReceivableCheckBo();
        sumBo.setSaleUserName("总计");
        sumBo.setReceivableAmount(list.stream().map(ReceivableCheckBo::getReceivableAmount).reduce(BigDecimal.ZERO, BigDecimal::add));
        sumBo.setAgeLess30Amount(list.stream().map(ReceivableCheckBo::getAgeLess30Amount).reduce(BigDecimal.ZERO, BigDecimal::add));
        sumBo.setAge30To60Amount(list.stream().map(ReceivableCheckBo::getAge30To60Amount).reduce(BigDecimal.ZERO, BigDecimal::add));
        sumBo.setAge60To90Amount(list.stream().map(ReceivableCheckBo::getAge60To90Amount).reduce(BigDecimal.ZERO, BigDecimal::add));
        sumBo.setAge90To120Amount(list.stream().map(ReceivableCheckBo::getAge90To120Amount).reduce(BigDecimal.ZERO, BigDecimal::add));
        sumBo.setAgeGreater120Amount(list.stream().map(ReceivableCheckBo::getAgeGreater120Amount).reduce(BigDecimal.ZERO, BigDecimal::add));
        list.add(sumBo);
        return list;
    }

    @Override
    public List<JSONObject> getReceivableAccounts(ReceivableAccountsSelectDto dto) {
        Integer year = dto.getYear();
        Assert.notEmpty(year, "年份不能为空");

        //获取一年里有交易的客户信息
        List<CustomerMonthlyAccounts> customerMonthlyAccounts = reportMapper.getCustomerMonthlyAccounts(year);
        Map<Long, List<CustomerMonthlyAccounts>> monthlyAccountsMap = customerMonthlyAccounts
                .stream().collect(Collectors.groupingBy(CustomerMonthlyAccounts::getBuyCorporationId));
        //获取年初额度
        Map<Long, CustomerPayableBalance> lastPayableBalanceMap = customerPayableBalanceService.mapKEntity(
                CustomerPayableBalance::getCustomerId,
                q -> q.eq(CustomerPayableBalance::getAccountDate, StrUtil.format("{}-12", year - 1))
        );

        //获取今年的调整信息
        Map<Long, List<CustomerPayableBalance>> payableBalanceListMap = customerPayableBalanceService.mapKGroup(
                CustomerPayableBalance::getCustomerId,
                q -> q.apply("YEAR(account_date) = {0}", year)
        );
        List<Long> customerIds = customerMonthlyAccounts.stream().map(CustomerMonthlyAccounts::getBuyCorporationId).distinct().collect(Collectors.toList());
        List<CustomerVo> customerList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(customerIds)) {
            customerList = customerService.getList(IWrapper.<CustomerVo>getWrapper().in("c", Customer::getId, customerIds));
            //赋值业务员信息
            UserUtil.assignmentNickName(customerList, CustomerVo::getUserId, CustomerVo::setUserName);
        }
        Map<Long, CustomerVo> customerMap = customerList.stream().collect(Collectors.toMap(CustomerVo::getId, Function.identity()));

        JSONObject totalBo = new JSONObject();
        totalBo.put("customerShortName", "总计");

        List<JSONObject> outList = new ArrayList<>();
        for (Long customerId : customerIds) {
            //获取年初信息
            BigDecimal yearBeginBalance = BigDecimal.ZERO;
            CustomerPayableBalance customerPayableBalance = lastPayableBalanceMap.get(customerId);
            if (ObjectUtil.isNotNull(customerPayableBalance)) {
                yearBeginBalance = customerPayableBalance.getBalance();
            }

            //获取每月信息
            List<CustomerMonthlyAccounts> monthlyAccountsList = monthlyAccountsMap.getOrDefault(customerId, new ArrayList<>());
            Map<Integer, CustomerMonthlyAccounts> itemMonthlyAccountsMap = monthlyAccountsList
                    .stream().collect(Collectors.toMap(CustomerMonthlyAccounts::getDataMonth, Function.identity()));

            //获取调整信息
            List<CustomerPayableBalance> payableBalanceList = payableBalanceListMap.getOrDefault(customerId, new ArrayList<>());
            Map<Date, CustomerPayableBalance> payableBalanceMap = payableBalanceList
                    .stream().collect(Collectors.toMap(CustomerPayableBalance::getAccountDate, Function.identity()));

            //上月余额
            BigDecimal lastBalanceAmount = yearBeginBalance;

            CustomerVo customer = customerMap.getOrDefault(customerId, new CustomerVo());

            JSONObject dataJson = new JSONObject();
            dataJson.put("customerId", customerId);
            dataJson.put("customerName", customer.getName());
            dataJson.put("customerShortName", customer.getShortName());
            dataJson.put("saleUserName", customer.getUserName());
            dataJson.put("yearBeginBalance", yearBeginBalance);
            for (int i = 1; i <= 12; i++) {
                //获取应收 已收
                BigDecimal receivableAmount = BigDecimal.ZERO;
                BigDecimal receivedAmount = BigDecimal.ZERO;
                CustomerMonthlyAccounts customerMonthlyAccounts1 = itemMonthlyAccountsMap.get(i);
                if (ObjectUtil.isNotEmpty(customerMonthlyAccounts1)) {
                    receivableAmount = customerMonthlyAccounts1.getTruckAmount();
                    //调整为负数
                    receivedAmount = BigDecimal.ZERO.subtract(customerMonthlyAccounts1.getClaimMonth());
                }

                //获取调整
                BigDecimal adjustAmount = BigDecimal.ZERO;
                CustomerPayableBalance customerPayableBalance1 = payableBalanceMap
                        .get(DateUtil.parse(StrUtil.format("{}-{}", year, i), "yyyy-MM"));
                if (ObjectUtil.isNotEmpty(customerPayableBalance1)) {
                    adjustAmount = customerPayableBalance1.getAdjust();
                }

                //计算余额
                BigDecimal balanceAmount = lastBalanceAmount.add(receivableAmount).add(receivedAmount).add(adjustAmount);

                //赋值上月余额
                lastBalanceAmount = balanceAmount;


                JSONObject itemJson = new JSONObject();
                itemJson.put("receivableAmount", receivableAmount);
                itemJson.put("receivedAmount", receivedAmount);
                itemJson.put("adjustAmount", adjustAmount);
                itemJson.put("balanceAmount", balanceAmount);
                dataJson.put("" + i, itemJson);

                //合计
                JSONObject totalItemBo = ObjectUtil.defaultIfNull(totalBo.getJSONObject("" + i), new JSONObject());
                totalItemBo.put("receivableAmount", ObjectUtil.defaultIfNull(totalItemBo.getBigDecimal("receivableAmount"), BigDecimal.ZERO).add(itemJson.getBigDecimal("receivableAmount")));
                totalItemBo.put("receivedAmount", ObjectUtil.defaultIfNull(totalItemBo.getBigDecimal("receivedAmount"), BigDecimal.ZERO).add(itemJson.getBigDecimal("receivedAmount")));
                totalItemBo.put("adjustAmount", ObjectUtil.defaultIfNull(totalItemBo.getBigDecimal("adjustAmount"), BigDecimal.ZERO).add(itemJson.getBigDecimal("adjustAmount")));
                totalItemBo.put("balanceAmount", ObjectUtil.defaultIfNull(totalItemBo.getBigDecimal("balanceAmount"), BigDecimal.ZERO).add(itemJson.getBigDecimal("balanceAmount")));
                totalBo.put("" + i, totalItemBo);
            }
            outList.add(dataJson);
        }

        //合计
        outList.add(totalBo);

        return outList;
    }

    /**
     * 每月1日0点1分 赋值客户上月应付余额
     */
//    @PostConstruct
    @Scheduled(cron = "0 1 0 1 * ?")
    public void autoSaveReceivableAccountsEnding() {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        SecurityUtils.setTenantId("000000");

        ReceivableAccountsSelectDto dto = new ReceivableAccountsSelectDto();
        DateTime lastMonth = DateUtil.offsetMonth(new Date(), -1);
        dto.setYear(DateUtil.year(lastMonth));
        List<JSONObject> receivableAccounts = getReceivableAccounts(dto);
        for (JSONObject receivableAccount : receivableAccounts) {
            int month = DateUtil.month(lastMonth) + 1;
            JSONObject item = receivableAccount.getJSONObject("" + month);
            Long customerId = receivableAccount.getLong("customerId");

            if (ObjectUtil.isEmpty(customerId)) {
                continue;
            }

            DateTime parse = DateUtil.parse(StrUtil.format("{}-{}", dto.getYear(), month), "yyyy-MM");

            CustomerPayableBalance customerPayableBalance = customerPayableBalanceService.getOne(q -> q
                    .eq(CustomerPayableBalance::getCustomerId, customerId)
                    .eq(CustomerPayableBalance::getAccountDate, parse)
            );
            if (ObjectUtil.isEmpty(customerPayableBalance)) {
                customerPayableBalance = new CustomerPayableBalance();
                customerPayableBalance.setCustomerId(customerId);
                customerPayableBalance.setAccountDate(parse);
            }
            customerPayableBalance.setBalance(item.getBigDecimal("balanceAmount"));
            customerPayableBalanceService.saveOrUpdate(customerPayableBalance);
        }

        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.clear();
    }

    @Override
    public List<UserPerformanceBo> userPerformanceReport(UserPerformanceSelectDto dto) {
        Assert.notEmpty(dto.getDataDate(), "日期过滤不能为空");
        List<UserPerformanceBo> userPerformanceBos = reportMapper.userPerformanceReport(dto.getDataDate(), dto.getUserId());
        //获取月份天数
        int month = DateUtil.month(dto.getDataDate()) + 1;
        int dayCount = DateUtil.lengthOfMonth(month, DateUtil.isLeapYear(DateUtil.year(dto.getDataDate())));

        //合计
        UserPerformanceBo totalBo = new UserPerformanceBo();
        totalBo.setUserName("合计");
        totalBo.setMonth(month);
        totalBo.setAmount(BigDecimal.ZERO);
        totalBo.setDayData(new JSONObject());

        for (UserPerformanceBo userPerformanceBo : userPerformanceBos) {
            userPerformanceBo.setMonth(month);

            JSONObject parse = JSONObject.parse(userPerformanceBo.getDayJson());

            JSONObject dayData = new JSONObject();
            for (int i = 1; i <= dayCount; i++) {
                dayData.put("" + i, parse.getOrDefault(i < 10 ? "0" + i : "" + i, 0));

                //计算合计
                JSONObject totalDayData = totalBo.getDayData();
                totalDayData.put("" + i, ObjectUtil.defaultIfNull(totalDayData.getBigDecimal("" + i), BigDecimal.ZERO).add(dayData.getBigDecimal("" + i)));
                totalBo.setDayData(totalDayData);
            }

            userPerformanceBo.setDayData(dayData);

            //计算合计
            totalBo.setAmount(totalBo.getAmount().add(ObjectUtil.defaultIfNull(userPerformanceBo.getAmount(), BigDecimal.ZERO)));
        }

        userPerformanceBos.add(totalBo);

        return userPerformanceBos;
    }
}
