package com.fjhx.form.service.impl;

import com.fjhx.common.entity.claim.dto.ClaimSelectDto;
import com.fjhx.flow.entity.flow.dto.FlowExampleSelectDto;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.form.mapper.StatisticsMapper;
import com.fjhx.form.service.StatisticsService;
import com.fjhx.mes.entity.production.dto.ProduceOrderSelectDto;
import com.fjhx.mes.service.production.ProduceOrderService;
import com.fjhx.purchase.entity.subscribe.dto.SubscribeDetailSelectDto;
import com.fjhx.purchase.service.subscribe.SubscribeDetailService;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.purchase.vo.EhsdPurchaseVo;
import com.fjhx.sale.entity.sale.dto.SaleQuotationSelectDto;
import com.fjhx.sale.service.claim.ClaimService;
import com.fjhx.sale.service.sale.SaleQuotationService;
import com.fjhx.wms.entity.stock.dto.StockSelectDto;
import com.fjhx.wms.entity.stock.dto.StockWaitSelectDto;
import com.fjhx.wms.service.stock.StockService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class StatisticsServiceImpl implements StatisticsService {
    @Autowired
    private FlowExampleService flowExampleService;
    @Autowired
    private StatisticsMapper statisticsMapper;
    @Autowired
    private SaleQuotationService saleQuotationService;
    @Autowired
    private ProduceOrderService produceOrderService;
    @Autowired
    private StockService stockService;
    @Autowired
    private StockWaitService stockWaitService;
    @Autowired
    private ClaimService claimService;
    @Resource
    private SubscribeDetailService subscribeDetailService;

    /**
     * 工作统计
     */
    @Override
    public Map<String, Long> workStatistics() {
        Long userId = SecurityUtils.getUserId();

        //权限过滤：子公司看自己的,总公司看全部
        List<Long> companyIds = SecurityUtils.getCompanyIds();

        //是否是总公司
        boolean isHeadOffice = companyIds.contains(100L);

        //统计待审批数据
        FlowExampleSelectDto flowExampleSelectDto = new FlowExampleSelectDto();
        flowExampleSelectDto.setStatus(1);
        long waitFlowCount = flowExampleService.getToBeProcessedPage(flowExampleSelectDto).getTotal();

        //待报价 总公司对内报价单 子公司 报价评估
        SaleQuotationSelectDto saleQuotationSelectDto = new SaleQuotationSelectDto();
        saleQuotationSelectDto.setIsEstimate(1);
        saleQuotationSelectDto.setQuotationStatus("0,1");
        long waitQuotationCount = saleQuotationService.getPage(saleQuotationSelectDto).getTotal();

        //待投产
        ProduceOrderSelectDto produceOrderSelectDto = new ProduceOrderSelectDto();
        produceOrderSelectDto.setIsProduce(0);
        long waitProduceCount = produceOrderService.getPage(produceOrderSelectDto).getTotal();

        //待入库
        StockWaitSelectDto stockWaitSelectDto = new StockWaitSelectDto();
        stockWaitSelectDto.setType(1);
        long waitInStockCount = stockWaitService.getPage(stockWaitSelectDto).getTotal();

        //待出库
        StockWaitSelectDto stockWaitSelectDto1 = new StockWaitSelectDto();
        stockWaitSelectDto1.setType(2);
        long waitOutStockCount = stockWaitService.getPage(stockWaitSelectDto1).getTotal();

        //到账认领待认领计数-总公司
        ClaimSelectDto claimSelectDto = new ClaimSelectDto();
        claimSelectDto.setConfirmStatus(0);
        long waitClaimCount = claimService.claimInfoConfirmPage(claimSelectDto).getTotal();

        //库存预警-子公司看
        long stockWarnCount = stockService.stockWarningPage(new StockSelectDto()).getTotal();

        //待采购
        SubscribeDetailSelectDto subscribeDto = new SubscribeDetailSelectDto();
        subscribeDto.setIsWaitPurchase(1);
        subscribeDto.setStatus("15,30");
        long waitPurchaseCount = subscribeDetailService.getPage(subscribeDto).getTotal();

        //总数
        long sumCount = waitFlowCount + waitQuotationCount + waitProduceCount + waitInStockCount + waitOutStockCount + waitPurchaseCount;

        if (isHeadOffice) {
            sumCount += waitClaimCount;//总公司待认领
        } else {
            sumCount += stockWarnCount;//子公司库存预警
        }

        //输出结果
        Map<String, Long> map = new HashMap<>();
        map.put("waitFlowCount", waitFlowCount);
        map.put("waitQuotationCount", waitQuotationCount);
        map.put("waitProduceCount", waitProduceCount);
        map.put("waitInStockCount", waitInStockCount);
        map.put("waitOutStockCount", waitOutStockCount);
        map.put("waitClaimCount", waitClaimCount);
        map.put("stockWarnCount", stockWarnCount);
        map.put("waitPurchaseCount", waitPurchaseCount);
        map.put("sumCount", sumCount);

        return map;
    }

    @Override
    public List<ContractVo> getRecentContractShipmentList() {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        //权限过滤：近期出货合同
        wrapper.in("c.company_id", SecurityUtils.getCompanyIds());


        wrapper.eq("c.status", 30);
        wrapper.ge("DATEDIFF(c.delivery_time,NOW())", 0);
        wrapper.le("DATEDIFF(c.delivery_time,NOW())", 7);
        List<ContractVo> contractSaitShipmentList = statisticsMapper.getRecentContractShipmentList(wrapper);
        return contractSaitShipmentList;
    }

    @Override
    public List<EhsdPurchaseVo> getRecentPurchaseArrivalList() {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        //权限过滤：近期出货合同
        wrapper.in("p.company_id", SecurityUtils.getCompanyIds());


        wrapper.ge("DATEDIFF(p.delivery_time,NOW())", 0);
        wrapper.le("DATEDIFF(p.delivery_time,NOW())", 7);
        wrapper.eq("p.status", 30);
        List<EhsdPurchaseVo> contractSaitShipmentList = statisticsMapper.getRecentPurchaseArrivalList(wrapper);
        return contractSaitShipmentList;
    }
}
