package com.fjhx.mes.service.production.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.enums.PushBusinessTypeEnum;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.MaterialBalanceBo;
import com.fjhx.mes.entity.production.dto.ProduceOrderSelectDto;
import com.fjhx.mes.entity.production.dto.ProductionOrderDto;
import com.fjhx.mes.entity.production.po.ProductionOrder;
import com.fjhx.mes.entity.production.po.ProductionOrderDetail;
import com.fjhx.mes.entity.production.po.ProductionTaskProgress;
import com.fjhx.mes.entity.production.vo.ProductionOrderDetailVo;
import com.fjhx.mes.entity.production.vo.ProductionOrderVo;
import com.fjhx.mes.entity.production.vo.ProductionProcessesVo;
import com.fjhx.mes.mapper.production.ProduceOrderMapper;
import com.fjhx.mes.mapper.production.ProductionProcessesMapper;
import com.fjhx.mes.service.material.MaterialPreparationService;
import com.fjhx.mes.service.production.ProduceOrderDetailService;
import com.fjhx.mes.service.production.ProduceOrderService;
import com.fjhx.mes.service.production.ProductionTaskProgressService;
import com.fjhx.purchase.service.subscribe.SubscribeDetailService;
import com.fjhx.sale.entity.arrival.po.ArrivalDetail;
import com.fjhx.sale.entity.arrival.vo.ArrivalDetailVo;
import com.fjhx.sale.entity.contract.dto.ContractSelectDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.contract.vo.ContractProductVo;
import com.fjhx.sale.entity.contract.vo.ContractVo;
import com.fjhx.sale.entity.purchase.po.EhsdPurchaseProduct;
import com.fjhx.sale.entity.purchase.vo.EhsdPurchaseProductVo;
import com.fjhx.sale.mapper.arrival.ArrivalDetailMapper;
import com.fjhx.sale.mapper.purchase.EhsdPurchaseProductMapper;
import com.fjhx.sale.service.contract.ContractProductBomService;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.sale.service.contract.ContractService;
import com.fjhx.socket.core.PushTypeEnum;
import com.fjhx.socket.core.WebSocketPush;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.tenant.utils.DeptUstil;
import com.fjhx.wms.entity.stock.emums.JournalType;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.entity.stock.po.StockWait;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.entity.stock.vo.StockJournalDetailsVo;
import com.fjhx.wms.mapper.stock.StockJournalDetailsMapper;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产订单 服务实现类
 * </p>
 *
 * @since 2024-01-10
 */
@Service
public class ProduceOrderServiceImpl extends ServiceImpl<ProduceOrderMapper, ProductionOrder> implements ProduceOrderService {

    @Autowired
    private ContractService contractService;
    @Autowired
    private ContractProductService contractProductService;
    @Autowired
    private ProduceOrderDetailService produceOrderDetailService;
    @Autowired
    private ISysDeptService sysDeptService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private MaterialPreparationService materialPreparationService;
    @Autowired
    private ProductionProcessesMapper productionProcessesMapper;
    @Autowired
    private ProductionTaskProgressService productionTaskProgressService;
    @Autowired
    private ContractProductBomService contractProductBomService;
    @Autowired
    private SubscribeDetailService subscribeDetailService;
    @Autowired
    private EhsdPurchaseProductMapper ehsdPurchaseProductMapper;
    @Autowired
    private StockJournalDetailsMapper stockJournalDetailsMapper;
    @Autowired
    private StockWaitService stockWaitService;
    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;
    @Autowired
    private ArrivalDetailMapper arrivalDetailMapper;
    @Autowired
    private SupplierInfoService supplierInfoService;

    @Override
    public Page<ProductionOrderVo> getPage(ProduceOrderSelectDto dto) {
        IWrapper<ProductionOrder> wrapper = getWrapper();
        wrapper.orderByDesc("po", ProductionOrder::getId);

        //生产状态
        String produceStatus = dto.getProduceStatus();
        if (ObjectUtil.isNotEmpty(produceStatus)) {
            wrapper.in(ProductionOrder::getProduceStatus, produceStatus.split(","));
        }
        //交期-时间范围过滤
        wrapper.ge("po", ProductionOrder::getDeliveryPeriod, dto.getStaDeliveryPeriod());
        wrapper.le("po", ProductionOrder::getDeliveryPeriod, dto.getEndDeliveryPeriod());
        //下单时间（创建时间）-时间范围过滤
        wrapper.ge("po", ProductionOrder::getCreateTime, dto.getBeginTime());
        wrapper.le("po", ProductionOrder::getCreateTime, dto.getEndTime());
        //关键字
        wrapper.keyword(dto.getKeyword(), new SqlField("po", ProductionOrder::getCode));

        //权限过滤：生产订单
        wrapper.in("po", ProductionOrder::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("po", ProductionOrder::getCompanyId, dto.getCompanyId());
        //业务公司过滤
        wrapper.eq("c.of_company_id", dto.getContractCompanyId());


        //是否逾期过滤
        if (ObjectUtil.isNotEmpty(dto.getIsOverdue())) {
            wrapper.eq("IF(IFNULL(po.finish_time,NOW())>po.delivery_period,1,0)", dto.getIsOverdue());
        }

        //是否投产过滤
        if (ObjectUtil.isNotEmpty(dto.getIsProduce())) {
            wrapper.eq("IF( po.produce_time IS NULL, 0, 1 )", dto.getIsProduce());
        }

        //订单类型检索
        if (ObjectUtil.isNotEmpty(dto.getContractType())) {
            wrapper.eq("c.contract_type", dto.getContractType());
        }

        //不显示指定类型数据
        String neProduceStatus = dto.getNeProduceStatus();
        if (ObjectUtil.isNotEmpty(neProduceStatus)) {
            wrapper.notIn(ProductionOrder::getProduceStatus, neProduceStatus.split(","));
        }

        //欠料状态过滤
        if (ObjectUtil.isNotEmpty(dto.getLackStatus())) {
            wrapper.apply("IFNULL( t2.lackStatus, 0 ) = {0}", dto.getLackStatus());
        }

        Page<ProductionOrderVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionOrderVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值订单明细信息
        List<Long> produceOrderIds = records.stream().map(ProductionOrder::getId).collect(Collectors.toList());
        List<ProductionOrderDetail> productionOrderDetailList = produceOrderDetailService.list(q -> q.in(ProductionOrderDetail::getProduceOrderId, produceOrderIds));
        List<ProductionOrderDetailVo> produceOrderDetailVos = BeanUtil.copyToList(productionOrderDetailList, ProductionOrderDetailVo.class);

        productInfoService.attributeAssign(produceOrderDetailVos, ProductionOrderDetail::getProductId, (item, product) -> {
            item.setProductCode(product.getCustomCode());
            item.setProductName(product.getName());
            item.setProductUnit(product.getUnit());
            item.setProductSpec(product.getSpec());
            item.setProductLength(product.getLength());
            item.setProductWidth(product.getWidth());
            item.setProductHeight(product.getHeight());
            item.setProductColor(product.getColor());
        });

        Map<Long, List<ProductionOrderDetailVo>> produceOrderDetailMap = produceOrderDetailVos.stream().collect(Collectors.groupingBy(ProductionOrderDetail::getProduceOrderId));

        //获取归属公司信息
        Map<Long, String> sysDeptMap = new HashMap<>();
        List<Long> companyIds = records.stream().map(ProductionOrder::getCompanyId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(companyIds)) {
            List<SysDept> deptList = sysDeptService.list();
            sysDeptMap = deptList.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));
        }


        for (ProductionOrderVo record : records) {
            record.setProduceOrderDetailList(produceOrderDetailMap.get(record.getId()));

            //赋值生产公司
            record.setCompanyName(sysDeptMap.get(record.getCompanyId()));

            record.setContractCompanyName(sysDeptMap.get(record.getContractCompanyId()));

        }

        return page;
    }

    @Override
    public List<ProductionOrderVo> getSchedulingList(ProduceOrderSelectDto dto) {
        //没传默认当前月
        if (ObjectUtil.isEmpty(dto.getBeginDate())) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
            dto.setBeginDate(sdf.format(new Date()));
        }

        IWrapper<ProductionOrder> wrapper = getWrapper();
        //排程时间过滤
        wrapper.and(q -> q
                .eq("DATE_FORMAT( po.create_time, '%Y-%m' )", dto.getBeginDate())
                .or().eq("DATE_FORMAT( po.delivery_period, '%Y-%m' )", dto.getBeginDate())
                .or(q1 -> q1
                        .le("DATE_FORMAT( po.create_time, '%Y-%m' )", dto.getBeginDate())
                        .ge("DATE_FORMAT( po.delivery_period, '%Y-%m' )", dto.getBeginDate())
                )
        );

        //过滤已投产的数据
        wrapper.isNotNull("po.produce_time");

        //是否逾期过滤
        if (ObjectUtil.isNotEmpty(dto.getIsOverdue())) {
            wrapper.eq("IF(IFNULL(po.finish_time,NOW())>po.delivery_period,1,0)", dto.getIsOverdue());
        }

        //权限过滤：生产排程

        wrapper.in("po", ProductInfo::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("po", ProductInfo::getCompanyId, dto.getCompanyId());


        wrapper.groupBy("po.id");
        wrapper.orderByAsc("po", ProductionOrder::getDeliveryPeriod);
        wrapper.orderByAsc("po", ProductionOrder::getCreateTime);
        wrapper.orderByAsc("po", ProductionOrder::getId);
        List<ProductionOrderVo> schedulingList = baseMapper.getSchedulingList(wrapper);

        //获取生产部门信息
        List<Long> companyIds = schedulingList.stream().map(ProductionOrderVo::getCompanyId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(companyIds)) {
            Map<Long, String> deptMap = DeptUstil.getDeptNameMap(companyIds);
            for (ProductionOrderVo productionOrderVo : schedulingList) {
                //赋值生产公司名称
                productionOrderVo.setCompanyName(deptMap.get(productionOrderVo.getCompanyId()));
            }
        }

        return schedulingList;
    }

    @Override
    @DSTransactional
    public void createOrder(ProductionOrderDto dto) {
        Long contractId = dto.getContractId();
        Assert.notEmpty(dto.getContractId(), "合同id不能为空！");
        //获取合同信息
        Contract contract = contractService.getById(contractId);
        Assert.notEmpty(contract, "查询不到合同信息！");

        //根据合同产品归属公司
        List<Long> pIds = contractProductService.listObject(ContractProduct::getProductId, q -> q.eq(ContractProduct::getContractId, contractId));
        List<Long> pCompanyIds = productInfoService.listObject(ProductInfo::getCompanyId, q -> q.in(ProductInfo::getId, pIds));
        pCompanyIds = pCompanyIds.stream().distinct().collect(Collectors.toList());
        for (Long pCompanyId : pCompanyIds) {
            ProductionOrderDto produceOrderDto = BeanUtil.copyProperties(dto, ProductionOrderDto.class);

            //赋值原订单信息
            ProductionOrder one = this.getOne(q -> q.eq(ProductionOrder::getContractId, contractId).eq(ProductionOrder::getCompanyId, pCompanyId));
            if (ObjectUtil.isNotEmpty(one)) {
                produceOrderDto.setId(one.getId());
            }

            //指定生产公司为 产品归属公司
            SysDept produceCompany = sysDeptService.getById(pCompanyId);
            Assert.notEmpty(produceCompany, "查询不到生产公司信息");

            produceOrderDto.setProduceCompanyId(pCompanyId);
            produceOrderDto.setCompanyId(pCompanyId);
            produceOrderDto.setConfirmStatus(0);

            produceOrderDto.setCode(contract.getCode());
            produceOrderDto.setProduceStatus(0);
            this.saveOrUpdate(produceOrderDto);

            //消息推送：给归属公司的 子公司总经理
            String title = String.format("您有新的生产订单，订单号【%s】，请及时跟进。", produceOrderDto.getCode());
            JSONObject msg = new JSONObject();
            msg.put("business_id", produceOrderDto.getId());
            List<Long> generalManagerIds = UserUtil.getUserIdsByRoleKey("subsidiary_gm", pCompanyId);
            for (Long generalManagerId : generalManagerIds) {
                WebSocketPush.byUser(PushTypeEnum.MESSAGE, generalManagerId, title, PushBusinessTypeEnum.NEW_PRODUCTION_ORDER.getType(), msg.toString());
            }
        }

        //修改状态为投产
        contractService.update(q -> q
                .eq(Contract::getId, contractId)
                .set(Contract::getOrderDistributeStatus, 1)
                .set(Contract::getOrderDistributeTime, new Date())
                .set(Contract::getRejectRemark, null)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 修改投产时间
     */
    @Override
    public void editProduceTime(ProduceOrderSelectDto dto) {
        Long orderId = dto.getId();
        Date produceTime = dto.getProduceTime();

        Assert.notEmpty(orderId, "投产id不能为空！");
        Assert.notEmpty(produceTime, "投产时间不能为空！");

        ProductionOrder productionOrder = this.getById(orderId);
        Assert.notEmpty(productionOrder, "查询不到订单信息！");

        productionOrder.setProduceTime(produceTime);

        this.updateById(productionOrder);
    }


    /**
     * 创建生产任务
     */
    private synchronized void createOrderDetails(ProductionOrder produceOrder) {
        Long orderDtoId = produceOrder.getId();
        Long contractId = produceOrder.getContractId();
        Long companyId = produceOrder.getCompanyId();

        Contract contract = contractService.getById(contractId);
        Assert.notEmpty(contract, "查询不到合同信息！");

        //获取目前生产订单任务编号最大计数
        Integer maxTaskCount = 0;
        ProductionOrderDetail one = produceOrderDetailService.getOne(q -> q.eq(ProductionOrderDetail::getProduceOrderId, produceOrder.getId()));
        if (ObjectUtil.isNotEmpty(one) && ObjectUtil.isNotEmpty(one.getCode())) {
            String code = one.getCode();
            int lastIndex = code.lastIndexOf("-");
            if (lastIndex >= 0) {
                maxTaskCount = Integer.valueOf(code.substring(lastIndex + 1));
            }
        }

        //创建生产订单明细
        List<ProductionOrderDetail> productionOrderDetailList = new ArrayList<>();
        List<ContractProductVo> contractProductList = contractProductService.getList(IWrapper.<ContractProduct>getWrapper()
                .eq("cp", ContractProduct::getContractId, contractId)
                //过滤当前生产公司的产品
                .eq("pi.company_id", produceOrder.getCompanyId())
        );

        //将被删除的任务作废
        List<Long> cpIds = contractProductList.stream().map(ContractProduct::getId).collect(Collectors.toList());
        produceOrderDetailService.update(q -> q
                .eq(ProductionOrderDetail::getProduceOrderId, orderDtoId)
                .eq(ProductionOrderDetail::getContractId, contractId)
                .notIn(ProductionOrderDetail::getContractDetailId, cpIds)
                .set(ProductionOrderDetail::getProduceStatus, 88)
        );

        for (ContractProduct contractProduct : contractProductList) {
            ProductionOrderDetail productionOrderDetail = produceOrderDetailService.getOne(q -> q
                    .eq(ProductionOrderDetail::getProduceOrderId, orderDtoId)
                    .eq(ProductionOrderDetail::getContractId, contractId)
                    .eq(ProductionOrderDetail::getContractDetailId, contractProduct.getId())
            );
            if (ObjectUtil.isEmpty(productionOrderDetail)) {
                productionOrderDetail = new ProductionOrderDetail();
                productionOrderDetail.setFinishQuantity(BigDecimal.ZERO);

                productionOrderDetail.setCode(StrUtil.format("{}-{}", produceOrder.getCode(), ++maxTaskCount));
            }
            productionOrderDetail.setProduceOrderId(orderDtoId);
            productionOrderDetail.setContractId(contractId);
            productionOrderDetail.setContractDetailId(contractProduct.getId());
            productionOrderDetail.setProductId(contractProduct.getProductId());
            productionOrderDetail.setQuantity(contractProduct.getQuantity());

            //修改状态
            if (productionOrderDetail.getFinishQuantity().compareTo(productionOrderDetail.getQuantity()) >= 0) {
                //生产完成
                productionOrderDetail.setProduceStatus(2);
            } else {
                //进行中
                productionOrderDetail.setProduceStatus(1);
            }
            if (productionOrderDetail.getFinishQuantity().compareTo(BigDecimal.ZERO) == 0) {
                //未开始
                productionOrderDetail.setProduceStatus(0);
            }

            productionOrderDetail.setProduceStatus(0);
            productionOrderDetail.setCompanyId(companyId);
            productionOrderDetailList.add(productionOrderDetail);
        }
        produceOrderDetailService.saveOrUpdateBatch(productionOrderDetailList);

        List<Long> productIds = productionOrderDetailList.stream().map(ProductionOrderDetail::getProductId).collect(Collectors.toList());

        //生成生产任务进度
        List<ProductionTaskProgress> productionTaskProgressList = new ArrayList<>();
        List<ProductionProcessesVo> processes = productionProcessesMapper.getProcessesByProductId(IWrapper.getWrapper().in("pi.id", productIds));
        Map<Long, List<ProductionProcessesVo>> processesMap = processes.stream().collect(Collectors.groupingBy(ProductionProcessesVo::getProductId));
        for (ProductionOrderDetail productionOrderDetail : productionOrderDetailList) {
            List<ProductionProcessesVo> productionProcessesVos = processesMap.getOrDefault(productionOrderDetail.getProductId(), new ArrayList<>());
            for (ProductionProcessesVo productionProcessesVo : productionProcessesVos) {
                ProductionTaskProgress prodTaskProgress = productionTaskProgressService.getOne(q -> q
                        .eq(ProductionTaskProgress::getTaskId, productionOrderDetail.getId())
                        .eq(ProductionTaskProgress::getProcessesId, productionProcessesVo.getId())
                );
                if (ObjectUtil.isEmpty(prodTaskProgress)) {
                    prodTaskProgress = new ProductionTaskProgress();
                    prodTaskProgress.setFinishQuantity(BigDecimal.ZERO);
                }
                prodTaskProgress.setTaskId(productionOrderDetail.getId());
                prodTaskProgress.setProcessesId(productionProcessesVo.getId());

                //如果是第一道工序，结存数量直接等于任务数量
                prodTaskProgress.setBalanceQuantity(BigDecimal.ZERO);
                if (ObjectUtil.equals(productionProcessesVo.getSourceProcessesId(), 1L)) {
                    BigDecimal balanceQuantity = productionOrderDetail.getQuantity().subtract(prodTaskProgress.getFinishQuantity());
                    if (balanceQuantity.compareTo(BigDecimal.ZERO) < 0) {
                        balanceQuantity = BigDecimal.ZERO;
                    }
                    prodTaskProgress.setBalanceQuantity(balanceQuantity);
                }

                productionTaskProgressList.add(prodTaskProgress);
            }
        }
        productionTaskProgressService.saveOrUpdateBatch(productionTaskProgressList);
    }

//    /**
//     * 创建生产备料
//     */
//    private synchronized void createMaterialPreparation(ProductionOrder productionOrder) {
//        Long contractId = productionOrder.getContractId();
//        Long companyId = productionOrder.getCompanyId();
//        //创建生产备料
//        List<ProductionOrderDetail> productionOrderDetailList = produceOrderDetailService.list(q -> q
//                .eq(ProductionOrderDetail::getProduceOrderId, productionOrder.getId())
//        );
//        //获取合同产品ids
//        List<Long> cpIds = productionOrderDetailList.stream()
//                .map(ProductionOrderDetail::getContractDetailId).collect(Collectors.toList());
//        Map<Long, List<ContractProductBom>> cpBMap = contractProductBomService.mapKGroup(ContractProductBom::getContractProductId,
//                q -> q.in(ContractProductBom::getContractProductId, cpIds)
//        );
//
//        //获取产品id
//        for (ProductionOrderDetail productionOrderDetail : productionOrderDetailList) {
//            productionOrderDetail.setProduceStatus(1);
//            //创建备料
//            List<ContractProductBom> contractProductBomList = cpBMap.get(productionOrderDetail.getContractDetailId());
//
//            List<MaterialPreparation> materialPreparationList = new ArrayList<>();
//            for (ContractProductBom bomDetail : contractProductBomList) {
//                BigDecimal multiply = bomDetail.getQuantity().multiply(productionOrderDetail.getQuantity());
//
//                MaterialPreparation materialPreparation = materialPreparationService.getOne(q -> q
//                        .eq(MaterialPreparation::getProductionOrderDetailId, productionOrderDetail.getId())
//                        .eq(MaterialPreparation::getMaterialId, bomDetail.getMaterialId())
//                );
//
//                if (ObjectUtil.isEmpty(materialPreparation)) {
//                    materialPreparation = new MaterialPreparation();
//                    materialPreparation.setStatus(0);
//                } else {
//                    if (materialPreparation.getQuantity().compareTo(multiply) != 0 && materialPreparation.getStatus() == 1) {
//                        materialPreparation.setStatus(0);
//                    }
//                }
//                materialPreparation.setContractId(contractId);
//                materialPreparation.setContractDetailId(productionOrderDetail.getContractDetailId());
//                materialPreparation.setProductionOrderId(productionOrder.getId());
//                materialPreparation.setProductionOrderDetailId(productionOrderDetail.getId());
//                materialPreparation.setMaterialId(bomDetail.getMaterialId());
//                materialPreparation.setQuantity(multiply);
//                materialPreparation.setCompanyId(companyId);
//                materialPreparationList.add(materialPreparation);
//            }
//            materialPreparationService.saveOrUpdateBatch(materialPreparationList);
//        }
//
//        produceOrderDetailService.updateBatchById(productionOrderDetailList);
//    }

    @DSTransactional
    @Override
    public synchronized void deliveryConfirm(ProductionOrderDto dto) {
        Long orderDtoId = dto.getId();
        this.updateById(dto);

        //============刷新生产任务明细关联的合同产品id=============
        List<ProductionOrderDetail> tempProdTaskList = new ArrayList<>();

        List<ProductionOrderDetail> prodTaskList = produceOrderDetailService.list(q -> q
                .eq(ProductionOrderDetail::getProduceOrderId, orderDtoId)
        );
        for (ProductionOrderDetail prodTask : prodTaskList) {
            Long tempOldId = prodTask.getContractDetailId();
            //遍历出最新一版id
            int recursion = 0;
            while (ObjectUtil.isNotEmpty(tempOldId)) {
                recursion++;
                Long finalTempOldId = tempOldId;
                ContractProduct cp = contractProductService.getOne(q -> q.eq(ContractProduct::getOldContractProductId, finalTempOldId));
                if (ObjectUtil.isEmpty(cp)) {
                    prodTask.setContractDetailId(tempOldId);
                    break;
                }
                tempOldId = cp.getId();

                //递归超过50次，报错防止死循环
                if (recursion > 50) {
                    throw new ServiceException("数据异常，请联系管理员!");
                }
            }

            ProductionOrderDetail tempTask = new ProductionOrderDetail();
            tempTask.setId(prodTask.getId());
            tempTask.setContractDetailId(prodTask.getContractDetailId());
            tempProdTaskList.add(tempTask);
        }
        produceOrderDetailService.updateBatchById(tempProdTaskList);
        //===================================================

        ProductionOrder productionOrder = this.getById(orderDtoId);
        Assert.notEmpty(productionOrder, "查询不到订单信息！");
        //驳回直接删除
        if (ObjectUtil.equals(dto.getConfirmStatus(), 2)) {
            this.removeById(orderDtoId);
            //赋值驳回原因
            Long contractId = productionOrder.getContractId();
            contractService.update(q -> q
                    .eq(Contract::getId, contractId)
                    .set(Contract::getOrderDistributeStatus, 2)
                    .set(Contract::getOrderDistributeTime, new Date())
                    .set(Contract::getRejectRemark, dto.getRejectRemark())
                    .set(BasePo::getUpdateTime, new Date())
                    .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
            );
            //消息推送：给归属公司的 子公司总经理
            String title = String.format("您下发的生产订单被驳回，订单号【%s】，请及时跟进。", productionOrder.getCode());
            JSONObject msg = new JSONObject();
            msg.put("business_id", productionOrder.getId());
            WebSocketPush.byUser(PushTypeEnum.MESSAGE, productionOrder.getCreateUser(), title, PushBusinessTypeEnum.PRODUCTION_ORDER_REJECT.getType(), msg.toString());
        } else {
            //当合同下的所有订单确认交期 才生成信息
            long count = this.count(q -> q
                    .eq(ProductionOrder::getContractId, productionOrder.getContractId())
                    .eq(ProductionOrder::getConfirmStatus, 0));

            Contract contract = contractService.getById(productionOrder.getContractId());
            Assert.notEmpty(contract, "查询不到合同信息！");
            //所有订单交期确认, 合同为已下发, 才生成数据
            if (count == 0 && ObjectUtil.equals(contract.getOrderDistributeStatus(), 1)) {

                //创建/更新 待采购
                contractService.createWaitMaterial(contract.getId());

                List<ProductionOrder> productionOrderList = this.list(q -> q.eq(ProductionOrder::getContractId, productionOrder.getContractId()));
                for (ProductionOrder productionOrder1 : productionOrderList) {

//                    if (produceOrderDetailService.count(q -> q.eq(ProductionOrderDetail::getProduceOrderId, productionOrder1.getId())) > 0) {
//                        //修改待采购
//                        updateMaterialWaitPurchase(productionOrder1.getCompanyId(), productionOrder1.getContractId());
//                    } else {
//                        //创建待采购
//                        createMaterialWaitPurchase(productionOrder1.getCompanyId(), productionOrder1.getContractId());
//                    }

                    //创建生产任务
                    createOrderDetails(productionOrder1);

//                    //生成待出库
//                    createStockWait(productionOrder1);

                    //创建完工入库
                    createOrEditProdCompletion(productionOrder1);

                    productionOrder1.setProduceStatus(1);
                    productionOrder1.setProduceTime(new Date());//投产时间直接当前值

                    this.updateById(productionOrder1);
                }
            }
        }
    }

    private void createOrEditProdCompletion(ProductionOrder productionOrder) {
        //添加一条完工入库类型的待入库数据
        StockWait stockWait = stockWaitService.getOne(q -> q
                .eq(StockWait::getProdOrderId, productionOrder.getId())
                .eq(StockWait::getBusinessType, JournalType.COMPLETION_IN.getDetailType())
        );
        if (ObjectUtil.isEmpty(stockWait)) {
            stockWait = new StockWait();
        }
        stockWait.setType(1);
        stockWait.setBusinessType(JournalType.COMPLETION_IN.getDetailType());
        stockWait.setBusinessId(productionOrder.getId());
        stockWait.setBusinessCode(productionOrder.getCode());
        stockWait.setStatus(0);
        stockWait.setCompanyId(productionOrder.getCompanyId());
        stockWait.setProdOrderId(productionOrder.getId());
        stockWaitService.saveOrUpdate(stockWait);

        List<ProductionOrderDetail> prodTaskList = produceOrderDetailService.list(q -> q.eq(ProductionOrderDetail::getProduceOrderId, productionOrder.getId()));

        List<Long> taskIds = prodTaskList.stream().map(ProductionOrderDetail::getId).collect(Collectors.toList());
        StockWait finalStockWait = stockWait;
        stockWaitDetailsService.remove(q -> q.eq(StockWaitDetails::getStockWaitId, finalStockWait.getId()).notIn(StockWaitDetails::getProdTaskId, taskIds));
        for (ProductionOrderDetail prodTask : prodTaskList) {

            StockWaitDetails stockWaitDetails = stockWaitDetailsService.getOne(q -> q.eq(StockWaitDetails::getProdTaskId, prodTask.getId()));
            if (ObjectUtil.isEmpty(stockWaitDetails)) {
                stockWaitDetails = new StockWaitDetails();
                stockWaitDetails.setReceiptQuantity(BigDecimal.ZERO);
            }

            stockWaitDetails.setStockWaitId(stockWait.getId());
            stockWaitDetails.setProductId(prodTask.getProductId());
            stockWaitDetails.setQuantity(prodTask.getQuantity());
            stockWaitDetails.setBusinessDetailsId(prodTask.getId());
            stockWaitDetails.setProdTaskId(prodTask.getId());
            stockWaitDetailsService.saveOrUpdate(stockWaitDetails);
        }


    }

//    private void createStockWait(ProductionOrder productionOrder) {
//        //生成待出库
//        StockWait stockWait = stockWaitService.getOne(q -> q
//                .eq(StockWait::getBusinessId, productionOrder.getId())
//                .eq(StockWait::getBusinessType, JournalType.PROD_OUT.getDetailType())
//        );
//
//        if (ObjectUtil.isEmpty(stockWait)) {
//            stockWait = new StockWait();
//            stockWait.setStatus(0);//待出库
//        } else {
//            stockWait.setStatus(1);//部分出库
//        }
//        stockWait.setCompanyId(productionOrder.getCompanyId());
//        stockWait.setBusinessId(productionOrder.getId());
//        stockWait.setType(2);//出库
//
//        stockWait.setBusinessType(JournalType.PROD_OUT.getDetailType());//生产任务出库
//        stockWait.setContractId(productionOrder.getContractId());
//        stockWait.setBusinessCode(productionOrder.getCode());
//        stockWait.setProdOrderId(productionOrder.getId());
//        stockWaitService.saveOrUpdate(stockWait);
//
//        List<ContractProductVo> contractProductList = contractProductService.getList(IWrapper.<ContractProduct>getWrapper()
//                .eq("cp", ContractProduct::getContractId, productionOrder.getContractId())
//                //过滤当前生产公司的产品
//                .eq("pi.company_id", productionOrder.getCompanyId())
//        );
//        List<Long> cpIds = contractProductList.stream().map(ContractProduct::getId).collect(Collectors.toList());
//
//        List<StockWaitDetails> stockWaitDetailsList = new ArrayList<>();
//        List<ContractProductBomVo> contractProductBomQuantitySum = contractProductBomService.getContractProductBomQuantitySum(IWrapper.getWrapper()
//                .eq("cpb.contract_id", productionOrder.getContractId())
//                .in("cp.id", cpIds)
//                .ne("cpb.type", 3)//过滤掉工序的数据
//        );
//        for (ContractProductBomVo contractProductBomVo : contractProductBomQuantitySum) {
//            //创建待出库明细
//            Long stockWaitId = stockWait.getId();
//            StockWaitDetails stockWaitDetails = stockWaitDetailsService.getOne(q -> q
//                    .eq(StockWaitDetails::getStockWaitId, stockWaitId)
//                    .eq(StockWaitDetails::getProductId, contractProductBomVo.getMaterialId())
//            );
//
//            if (ObjectUtil.isEmpty(stockWaitDetails)) {
//                stockWaitDetails = new StockWaitDetails();
//                stockWaitDetails.setReceiptQuantity(BigDecimal.ZERO);
//            }
//
//            stockWaitDetails.setStockWaitId(stockWaitId);
//            stockWaitDetails.setProductId(contractProductBomVo.getMaterialId());
//            stockWaitDetails.setQuantity(contractProductBomVo.getQuantity());
//            stockWaitDetailsList.add(stockWaitDetails);
//        }
//        stockWaitDetailsService.saveOrUpdateBatch(stockWaitDetailsList);
//    }

    @Override
    public void editDeliveryPeriod(ProductionOrderDto dto) {
        this.update(q -> q
                .eq(ProductionOrder::getContractId, dto.getContractId())
                .set(ProductionOrder::getDeliveryPeriod, dto.getDeliveryPeriod())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public ProductionOrderVo detail(Long id) {
        ProductionOrder productionOrder = this.getById(id);
        Assert.notEmpty(productionOrder, "查询不到生产订单信息");
        ProductionOrderVo productionOrderVo = BeanUtil.copyProperties(productionOrder, ProductionOrderVo.class);

        //采购记录
        List<EhsdPurchaseProductVo> ehsdPurchaseProductVos = ehsdPurchaseProductMapper.getList(IWrapper.<EhsdPurchaseProduct>getWrapper()
                .eq("epp", EhsdPurchaseProduct::getContractId, productionOrder.getContractId())
        );
        //赋值卖方信息
        supplierInfoService.attributeAssign(ehsdPurchaseProductVos, EhsdPurchaseProductVo::getSellCorporationId, (item, supplier) -> {
            item.setSellCorporationName(supplier.getName());
        });
        //赋值产品信息
        productInfoService.attributeAssign(ehsdPurchaseProductVos, EhsdPurchaseProductVo::getProductId, (detail, product) -> {
            detail.setProductCode(product.getCustomCode());
            detail.setProductName(product.getName());
            detail.setProductSpec(product.getSpec());
            detail.setProductLength(product.getLength());
            detail.setProductWidth(product.getWidth());
            detail.setProductHeight(product.getHeight());
            detail.setProductColor(product.getColor());
        });

        //出入库流水记录
        List<StockJournalDetailsVo> stockJournalDetailsVos = stockJournalDetailsMapper.getList(IWrapper.<StockJournalDetails>getWrapper()
                .eq("sj", StockJournalDetails::getContractId, productionOrder.getContractId())
        );
        //赋值产品信息
        productInfoService.attributeAssign(stockJournalDetailsVos, StockJournalDetailsVo::getProductId, (detail, product) -> {
            detail.setProductCode(product.getCustomCode());
            detail.setProductName(product.getName());
            detail.setProductUnit(product.getUnit());
            detail.setProductSpec(product.getSpec());
            detail.setProductLength(product.getLength());
            detail.setProductWidth(product.getWidth());
            detail.setProductHeight(product.getHeight());
            detail.setProductColor(product.getColor());
        });

        //到货通知记录
        List<Long> pIds = ehsdPurchaseProductVos.stream().map(item -> item.getPurchaseId()).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(pIds)) {
            List<ArrivalDetailVo> arrivalDetailVos = arrivalDetailMapper.getList(IWrapper.<ArrivalDetail>getWrapper()
                    .in("ad.purchase_id", pIds)
            );

            //赋值结果
            productionOrderVo.setArrivalDetailList(arrivalDetailVos);
        }

        //赋值结果
        productionOrderVo.setPurchaseProductList(ehsdPurchaseProductVos);
        productionOrderVo.setStockJournalDetailsList(stockJournalDetailsVos);

        return productionOrderVo;
    }

    @Override
    public List<MaterialBalanceBo> materialBalanceList(Long id) {
        List<MaterialBalanceBo> materialBalanceBos = baseMapper.materialBalanceList(id);

        productInfoService.attributeAssign(materialBalanceBos, MaterialBalanceBo::getMaterialId, (item, material) -> {
            item.setMaterialCode(material.getCustomCode());
            item.setMaterialName(material.getName());
        });

        return materialBalanceBos;
    }

    @Override
    public ContractVo contractInfo(Long id) {
        ProductionOrder productionOrder = this.getById(id);

        ContractSelectDto dto = new ContractSelectDto();
        dto.setId(productionOrder.getContractId());
        dto.setProdCompanyId(productionOrder.getCompanyId());

        return contractService.detail(dto);
    }

    /**
     * 产品工艺校验
     */
    @Override
    public void notTechnologyCheck(Long orderId) {
        ProductionOrder poById = this.getById(orderId);
        Assert.notEmpty(poById, "查询不到生产订单信息！");

        //获取订单所在公司自己的产品
        List<Long> pIds = contractProductService.listObject(ContractProduct::getProductId, q -> q.eq(ContractProduct::getContractId, poById.getContractId()));
        List<ProductInfo> list = productInfoService.list(q -> q
                .in(ProductInfo::getId, pIds)
                .eq(ProductInfo::getCompanyId, poById.getCompanyId())
                .isNull(ProductInfo::getTechnologyId)
        );

        if (ObjectUtil.isNotEmpty(list)) {
            String collect = list.stream().map(item -> {
                String productName = ObjectUtil.defaultIfNull(item.getName(), "");
                String productCode = ObjectUtil.defaultIfNull(item.getCustomCode(), "");

                String productLength = ObjectUtil.isNull(item.getLength()) ? "" : item.getLength().toString();
                String productWidth = ObjectUtil.isNull(item.getWidth()) ? "" : item.getWidth().toString();
                String productHeight = ObjectUtil.isNull(item.getHeight()) ? "" : item.getHeight().toString();


                return StrUtil.format("{}({}[{}*{}*{}])", productName, productCode, productLength, productWidth, productHeight);
            }).collect(Collectors.joining("，"));
            throw new ServiceException("以下产品未配置工艺线路：" + collect);
        }
    }

}
