package com.fjhx.mes.service.production.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.flow.entity.flow.dto.FlowResult;
import com.fjhx.flow.entity.flow.dto.InitiateDto;
import com.fjhx.flow.service.flow.FlowProcessService;
import com.fjhx.mes.entity.production.dto.ProductionBeginConfirmDto;
import com.fjhx.mes.entity.production.po.ProductionBeginConfirm;
import com.fjhx.mes.entity.production.po.ProductionOrder;
import com.fjhx.mes.entity.production.vo.ProductionBeginConfirmVo;
import com.fjhx.mes.mapper.production.ProductionBeginConfirmMapper;
import com.fjhx.mes.service.production.ProduceOrderService;
import com.fjhx.mes.service.production.ProductionBeginConfirmService;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.service.contract.ContractService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产开工确认 服务实现类
 * </p>
 *
 * @author
 * @since 2024-06-19
 */
@Service
public class ProductionBeginConfirmServiceImpl extends ServiceImpl<ProductionBeginConfirmMapper, ProductionBeginConfirm> implements ProductionBeginConfirmService {

    @Resource
    private FlowProcessService flowProcessService;
    @Resource
    private ProduceOrderService produceOrderService;
    @Resource
    private ContractService contractService;

    @Override
    public FlowResult add(ProductionBeginConfirmDto dto) {
        Assert.notEmpty(dto.getProdOrderId(), "生产订单Id不能为空！");
        ProductionOrder orderById = produceOrderService.getById(dto.getProdOrderId());
        Assert.notEmpty(orderById, "查询不到生产信息！");
        Contract contractById = contractService.getById(orderById.getContractId());
        Assert.notEmpty(contractById, "查询不到合同信息！");

        JSONObject startData = (JSONObject) JSONObject.toJSON(dto);
        startData.put("salesmanId", contractById.getSalesmanId());

        InitiateDto initiateDto = new InitiateDto();
        initiateDto.setFlowKey("prod_begin_confirm_flow");
        initiateDto.setData(startData);
        return flowProcessService.initiate(initiateDto);
    }

    @Override
    public ProductionBeginConfirmVo detail(Long id) {
        ProductionBeginConfirmVo result = BeanUtil.copyProperties(this.getById(id), ProductionBeginConfirmVo.class);
        Assert.notEmpty(result, "无法查询到开工确认信息！");
        result.setHistoryList(getHistoryList(result.getProdOrderId()).stream().filter(item -> !item.getId().equals(id)).collect(Collectors.toList()));
        return result;
    }

    @Override
    public List<ProductionBeginConfirm> getHistoryList(Long orderId) {
        return this.list(q -> q.eq(ProductionBeginConfirm::getProdOrderId, orderId));
    }

}
