package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.invoice.dto.InvoiceApplyDto;
import com.fjhx.oa.entity.invoice.po.InvoiceApply;
import com.fjhx.oa.service.invoice.InvoiceApplyService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class InvoiceApplyFlow extends FlowDelegate {

    @Autowired
    private InvoiceApplyService invoiceApplyService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "invoice_apply_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        InvoiceApplyDto dto = submitData.toJavaObject(InvoiceApplyDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.INVOICE_APPLY.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        invoiceApplyService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        invoiceApplyService.update(q -> q
                .eq(InvoiceApply::getId, businessId)
                .set(InvoiceApply::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        invoiceApplyService.update(q -> q
                .eq(InvoiceApply::getId, businessId)
                .set(InvoiceApply::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        invoiceApplyService.cancellation(businessId);
    }
}
